/**************************************************************************************************
	$Header: /pub/cvsroot/mydns/src/lib/rr.c,v 1.29 2003/03/15 00:30:54 bboy Exp $

	Copyright (C) 2002  Don Moore <bboy@bboy.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at Your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**************************************************************************************************/

#include "mydns.h"

char mydns_rr_table_name[PATH_MAX] = MYDNS_RR_TABLE;

/* Optional columns */
int mydns_rr_use_active = 0;
int mydns_rr_use_stamp = 0;


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	MYDNS_RR_COUNT
	Returns the number of records in the rr table.
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
long
mydns_rr_count(sql)
	SQL *sql;
{
	return sql_count(sql, "SELECT COUNT(*) FROM %s", mydns_rr_table_name);
}
/*--- mydns_rr_count() --------------------------------------------------------------------------*/


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	MYDNS_SET_RR_TABLE_NAME
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void
mydns_set_rr_table_name(name)
	char *name;
{
	if (!name)
		strncpy(mydns_rr_table_name, MYDNS_RR_TABLE, sizeof(mydns_rr_table_name)-1);
	else
		strncpy(mydns_rr_table_name, name, sizeof(mydns_rr_table_name)-1);
}
/*--- mydns_set_rr_table_name() -----------------------------------------------------------------*/


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	MYDNS_RR_GET_TYPE
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
inline dns_qtype_t
mydns_rr_get_type(const char *type)
{
	switch (toupper(*type))
	{
		case 'A':
			if (type[1] == 'A')			/* `AAAA' */
				return DNS_T_AAAA;
#if ALIAS_ENABLED
			else if (type[1] == 'L')
				return DNS_T_ALIAS;
#endif
			else								/* `A' */
				return DNS_T_A;

		case 'C': return DNS_T_CNAME;
		case 'M': return DNS_T_MX;
		case 'N': return DNS_T_NS;
		case 'T': return DNS_T_TXT;
		case 'P': return DNS_T_PTR;
		case 'S': return DNS_T_SRV;
	}
	return 0;
}
/*--- mydns_rr_get_type() -----------------------------------------------------------------------*/


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	MYDNS_PARSE_RR
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
inline MYDNS_RR *
mydns_parse_rr(SQL_ROW row, const char *origin)
{
	MYDNS_RR *rv;

	if ((rv = (MYDNS_RR *)calloc(1, sizeof(MYDNS_RR))))
	{
		rv->next = NULL;

		rv->id = atou(row[0]);
		rv->zone = atou(row[1]);
		strncpy(rv->name, row[2], sizeof(rv->name)-1);
		strncpy(rv->data, row[3], sizeof(rv->data)-1);
		rv->aux = atou(row[4]);
		rv->ttl = atou(row[5]);
		rv->type = mydns_rr_get_type(row[6]);
#if ALIAS_ENABLED
		if (rv->type == DNS_T_ALIAS)
		{
			rv->type = DNS_T_A;
			rv->alias = 1;
		}
		else
			rv->alias = 0;
#endif

		/* Append origin to data if it's not there for these types: */
		if (origin)
			switch (rv->type)
			{
				case DNS_T_CNAME:
				case DNS_T_MX:
				case DNS_T_NS:
				case DNS_T_SRV:
					{
						int datalen = strlen(rv->data);
						if (rv->data[datalen-1] != '.')
						{
							strncat(rv->data, ".", sizeof(rv->data) - datalen - 1);
							strncat(rv->data, origin, sizeof(rv->data) - datalen - 2);
						}
					}
					break;
				default: break;
			}

		/* SRV records contain two unsigned 16-bit integers in the "data" field before
			the target, 'srv_weight' and 'srv_port' - parse them and make "data" contain
			only the target.  Also, make sure 'aux' fits into 16 bits,
			clamping values above 65535. */
		if (rv->type == DNS_T_SRV)
		{
			char *weight, *port, *target;

			/* Clamp 'aux' if necessary */
			if (rv->aux > 65535)
				rv->aux = 65535;

			/* Parse weight (into srv_weight), port (into srv_port), and target */
			target = rv->data;
			if ((weight = strsep(&target, " \t")))
			{
				rv->srv_weight = atoi(weight);
				if ((port = strsep(&target, " \t")))
					rv->srv_port = atoi(port);
				memmove(rv->data, target, strlen(target)+1);
			}
		}
	}
	return (rv);
}
/*-----------------------------------------------------------------------------------------------*/


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	MYDNS_RR_DUP
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
MYDNS_RR *
mydns_rr_dup(start, recurse)
	MYDNS_RR *start;
	int recurse;
{
	register MYDNS_RR *first = NULL, *last = NULL, *rr, *s, *tmp;

	for (s = start; s; s = tmp)
	{
		tmp = s->next;

		if ((rr = (MYDNS_RR *)malloc(sizeof(MYDNS_RR))))
		{
			rr->id = s->id;
			rr->zone = s->zone;
			strncpy(rr->name, s->name, sizeof(rr->name)-1);
			rr->type = s->type;
			strncpy(rr->data, s->data, sizeof(rr->data)-1);
			rr->aux = s->aux;
			rr->ttl = s->ttl;
#if ALIAS_ENABLED
			rr->alias = s->alias;
#endif

			rr->srv_weight = s->srv_weight;
			rr->srv_port = s->srv_port;

			rr->next = NULL;
			if (recurse)
			{
				if (!first) first = rr;
				if (last) last->next = rr;
				last = rr;
			}
			else
				return (rr);
		}
	}
	return (first);
}
/*-----------------------------------------------------------------------------------------------*/


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	MYDNS_RR_SIZE
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
inline size_t
mydns_rr_size(first)
	MYDNS_RR *first;
{
	register MYDNS_RR *p;
	register size_t size = 0;

	for (p = first; p; p = p->next)
		size += sizeof(MYDNS_RR) + strlen(p->name) + strlen(p->data);

	return (size);
}
/*-----------------------------------------------------------------------------------------------*/


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	_MYDNS_RR_FREE
	Frees the pointed-to structure.	Don't call this function directly, call the macro.
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
inline void
_mydns_rr_free(first)
	MYDNS_RR *first;
{
	register MYDNS_RR *p, *tmp;

	for (p = first; p; p = tmp)
	{
		tmp = p->next;

		free(p);
	}
}
/*-----------------------------------------------------------------------------------------------*/


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	MYDNS_RR_LOAD
	Returns 0 on success or nonzero if an error occurred.
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
int
mydns_rr_load(SQL *sql, MYDNS_RR **rptr, u_int32_t zone, dns_qtype_t type, char *name, char *origin)
{
	MYDNS_RR *first = NULL, *last = NULL;
	size_t	namelen, querylen;
	u_char	query[DNS_QUERYBUFSIZ],
				namequery[DNS_MAXNAMELEN + DNS_MAXNAMELEN + DNS_MAXNAMELEN + 25];
	u_char	*wheretype;
	u_char	esc[DNS_MAXNAMELEN + DNS_MAXNAMELEN + 1];
	char		*cp;
	SQL_RES	*res;
	SQL_ROW	row;

	if (rptr) *rptr = NULL;

	/* Verify args */
	if (!sql || !rptr || !name)
	{
		errno = EINVAL;
		return (-1);
	}

	/* Get the type='XX' part of the WHERE clause */
	switch (type)
	{
#if ALIAS_ENABLED
		case DNS_T_A:		wheretype = " AND (type='A' OR type='ALIAS')"; break;
#else
		case DNS_T_A:		wheretype = " AND type='A'"; break;
#endif
		case DNS_T_AAAA:	wheretype = " AND type='AAAA'"; break;
		case DNS_T_CNAME:	wheretype = " AND type='CNAME'"; break;
		case DNS_T_MX:		wheretype = " AND type='MX'"; break;
		case DNS_T_NS:		wheretype = " AND type='NS'"; break;
		case DNS_T_PTR:	wheretype = " AND type='PTR'"; break;
		case DNS_T_SOA:	wheretype = " AND type='SOA'"; break;
		case DNS_T_TXT:	wheretype = " AND type='TXT'"; break;
		case DNS_T_SRV:	wheretype = " AND type='SRV'"; break;
		case DNS_T_ANY:	wheretype = ""; break;
		default:
			errno = EINVAL;
			return (-1);
	}

	/* Construct query */
	namelen = strlen(name);
	if (namelen + namelen + 1 > sizeof(esc))
	{
		errno = EINVAL;
		return (-1);
	}
	sql_escstr(sql, esc, name, namelen);
	if (origin)
	{
		u_char origin_esc[DNS_MAXNAMELEN + DNS_MAXNAMELEN + 1];
	
		sql_escstr(sql, origin_esc, origin, strlen(origin));

		if (!namelen)
			snprintf(namequery, sizeof(namequery), "(NAME='' OR name='%s')", origin_esc);
		else
			snprintf(namequery, sizeof(namequery),
					   "(NAME='%s' OR name='%s.%s')", esc, esc, origin_esc);
	}
	else
		snprintf(namequery, sizeof(namequery), "NAME='%s'", esc);

	querylen = snprintf(query, sizeof(query),
		"SELECT "MYDNS_RR_FIELDS" FROM %s WHERE "
			"zone=%u%s AND %s%s", mydns_rr_table_name, zone, wheretype, namequery,
			(mydns_rr_use_active) ? " AND active=1" : "");

	/* Submit query */
	if (!(res = sql_query(sql, query, querylen)))
		return (-1);

	/* Add results to list */
	while ((row = sql_getrow(res)))
	{
		MYDNS_RR *new;

		if (!(new = mydns_parse_rr(row, origin)))
		{
			sql_free(res);
			return (-1);
		}

		/* Always trim origin from name (XXX: Why? When did I add this?) */
		if (origin && (cp = strstr(new->name, origin)))
			*cp = '\0';

		if (!first) first = new;
		if (last) last->next = new;
		last = new;
	}

	*rptr = first;
	sql_free(res);
	return (0);
}
/*-----------------------------------------------------------------------------------------------*/

/* vi:set ts=3: */
